/*
 * CONSTANT.INC
 * ============
 *
 * Changing any values in this file will probably cause things to
 * break.
 */

if !def(_CONSTANT_INC_)
def _CONSTANT_INC_ equ 1

if !def(_CONFIG_INC_) ; wrap in guard since CONFIG.INC doesn't have its own.
include "CONFIG.INC"
def _CONFIG_INC_ equ 1
endc; _CONFIG_INC_

include "GBDOS.INC"

def GBDOS_MASK_ROM equ   ((GBDOS_MAJOR_RELEASE_NUMBER & %00001111) << 4) \
                       + ((GBDOS_MINOR_RELEASE_NUMBER & %00001111) << 0)
println "{GBDOS_VERSION_STRING}"

def VOLUME_LABEL equs strsub("{GBDOS_NAME_STRING}           ", 0, 11)

def CART_SIZE equ $8000 * (1 << ROM_SIZE)

def ROM_BANK_COUNT equ 2 << ROM_SIZE

if MEMORY_BANK_CONTROLLER == 0
def CARTRIDGE_TYPE equ $00
if ROM_BANK_COUNT > 2
fail "ROM too big for MBC selected in CONFIG.INC"
endc
elif MEMORY_BANK_CONTROLLER == 3
def CARTRIDGE_TYPE equ $11
if ROM_BANK_COUNT > $80
fail "ROM too big for MBC selected in CONFIG.INC"
endc
elif MEMORY_BANK_CONTROLLER == 5
def CARTRIDGE_TYPE equ $19
if ROM_BANK_COUNT > $200
fail "ROM too big for MBC selected in CONFIG.INC"
endc
else
fail "Invalid Memory Bank Controller selection in CONFIG.INC"
endc

def VHD_FOOTER_SIZE equ 512;
def VHD_ADDRESS equ $8000 - VHD_FOOTER_SIZE
def VHD_BANK equ ROM_BANK_COUNT - 1

def IMAGE_SIZE equ CART_SIZE - VHD_FOOTER_SIZE

def BYTES_PER_SECTOR equ 512 ; WARNING: several places in the drivers assume that this is 512. DO NOT CHANGE!

def SECTORS_PER_CLUSTER equ 1 ; WARNING: some other calculations assume that this is 1. DO NOT CHANGE!

def ROOT_ENTRY_COUNT equ 32

def ROOT_DIRECTORY_SECTOR_COUNT equ ((ROOT_ENTRY_COUNT * 32) + (BYTES_PER_SECTOR - 1)) / BYTES_PER_SECTOR

def RESERVED_SECTOR_COUNT equ 1

; Make sure that the formatted space is small enough to allow the root directory to be accessible without banking
if (IMAGE_SIZE / BYTES_PER_SECTOR) * 2 + (ROOT_DIRECTORY_SECTOR_COUNT + RESERVED_SECTOR_COUNT) * BYTES_PER_SECTOR > $8000
def PARTITION_SIZE equ (($8000 - ((ROOT_DIRECTORY_SECTOR_COUNT + RESERVED_SECTOR_COUNT) * BYTES_PER_SECTOR)) / 2) * BYTES_PER_SECTOR
println "Shrinking fomatted area to fit root directory on ROM bank 1."
else
def PARTITION_SIZE equ IMAGE_SIZE
endc
println strfmt("Formatted space: %f%%", mul(div(PARTITION_SIZE, CART_SIZE), 100.0))

def TOTAL_SECTOR_COUNT equ PARTITION_SIZE / BYTES_PER_SECTOR

def NUMBER_OF_FATS equ 1

def FAT_SIZE equ ceil(div(TOTAL_SECTOR_COUNT * 2, BYTES_PER_SECTOR)) >> 16
if FAT_SIZE > 61
fail "Large FATs are not yet handled. Try reducing ROM size in CONFIG.INC"
endc

def DATA_SECTOR_COUNT equ TOTAL_SECTOR_COUNT - (RESERVED_SECTOR_COUNT + (NUMBER_OF_FATS * FAT_SIZE) + ROOT_DIRECTORY_SECTOR_COUNT)

def CLUSTER_COUNT equ DATA_SECTOR_COUNT / SECTORS_PER_CLUSTER

print strfmt("%u Clusters: ", CLUSTER_COUNT)
if CLUSTER_COUNT < 4085
def FAT_TYPE equ 12
println "Using FAT12."
elif CLUSTER_COUNT == 4085
def FAT_TYPE equ 16
println "Using FAT16."
warn "Volumes with 4085 clusters may not behave as intended and should be avoided."
elif CLUSTER_COUNT < 65525
def FAT_TYPE equ 16
println "Using FAT16."
else
fail "Volume too large: FAT32 not supported."
endc

endc; _CONSTANT_INC_
