include "GBDOS.INC"

section "kernel", rom0[$0000]
load "kernel_ram", wram0[$C000]

kernel_entry:
	ld a, %10010001
	ldh [$FF40], a ; r_LCDC

	; set scroll
	xor a
	ldh [$FF42], a ; r_SCY
	ldh [$FF43], a ; r_SCX

	; set palettes
	; dmg
	ld a, %00001111 ; WWBB
	ldh [$FF47], a ; r_BGP
	; cgb
	ld a, %10000000 | $00 ; auto-increment + address 0
	ldh [$FF68], a ; r_BGPI
	ld b, 8 ; 8 palettes
.cgb_pal_loop:
    ; XBBBBBGG GGGRRRRR little-endian
    call sub_vram_wait
    xor a
    ldh [$FF69], a ; color0 low
    ld a, %01111100
    ldh [$FF69], a ; color0 high
    call sub_vram_wait
    xor a
    ldh [$FF69], a ; color1 low
    ld a, %01111100
    ldh [$FF69], a ; color1 high
    call sub_vram_wait
    ld a, $FF
    ldh [$FF69], a ; color2 low  ; r_BGPD
    ldh [$FF69], a ; color2 high
    ldh [$FF69], a ; color3 low
    ldh [$FF69], a ; color3 high
    dec b
    ld a, b
    or a
    jr nz, .cgb_pal_loop


    call sub_load_font

    ld l, " "
    ld de, $9800
    ld bc, 32 * 32
    call sub_vram_fill

    ld hl, version_string
    ld de, $0605 ; row 6 col 5
    call sub_vram_strcpy

    ld hl, message_string
    ld de, $0800
    call sub_vram_strcpy

    ld hl, copyright_string
    ld de, $0900
    call sub_vram_strcpy

    ld hl, license_string
    ld de, $0A00
    call sub_vram_strcpy

lock:
    ei
    nop
    halt
    nop
    jr lock

sub_load_font:
/*
 * Unpacks font tile data
 *
 * Clobbers all
 */
    ld hl, font_data
    ld de, $8000 + (" " * 16)
    ld bc, font_data_end - font_data
.copy_loop:

    push bc

    ld a, [hl]
    swap a
    and %00000111
    push hl
    ld hl, expansion_table
    ld c, a
    ld b, 0
    add hl, bc
    call sub_vram_wait
    ld a, [hl]
    pop hl
    ld [de], a
    inc de
    ld [de], a
    inc de

    ld a, [hl+]
    and %00000111
    push hl
    ld hl, expansion_table
    ld c, a
    ld b, 0
    add hl, bc
    call sub_vram_wait
    ld a, [hl]
    pop hl
    ld [de], a
    inc de
    ld [de], a
    inc de

    pop bc
    dec bc
    ld a, b
    or c
    jr nz, .copy_loop
    ret

sub_vram_copy:
/*
 * Copies bytes to or from vram.
 *
 * BC: byte count (length)
 * DE: destination
 * HL: source
 * Clobbers A
 */
    call sub_vram_wait
    ld a, [hl+]
    ld [de], a
    inc de
    dec bc
    ld a, b
    or c
    jr nz, sub_vram_copy
    ret

sub_vram_fill:
/*
 * Fill vram with a value
 *
 * BC: byte count (length)
 * DE: destination
 * L: value
 * Clobbers A
 */
    call sub_vram_wait
    ld a, l
    ld [de], a
    inc de
    dec bc
    ld a, b
    or c
    ret z
    jr sub_vram_fill

sub_vram_strcpy:
/*
 * Copy a null-terminated string to bg at a given row/column
 *
 * D: row
 * E: column
 * HL: source
 * Clobbers all
 */
    ld c, e
    ld e, d
    ld d, 0
    ld a, 5
.mul_loop:
    sla e
    rl d
    dec a
    or a
    jr nz, .mul_loop
    ld a, c
    add e
    ld e, a
    push hl
    ld hl, $9800
    add hl, de
    ld d, h
    ld e, l
    pop hl
.copy_loop:
    call sub_vram_wait
    ldi a, [hl]
    or a
    ret z
    ld [de], a
    inc de
    jr .copy_loop


sub_vram_wait:
/*
 * Wait for vram to be available.
 * Busywaits like this are usually not great, but this is to be used
 * primarily for diplaying messages about kernel errors, so
 * performance is not a concern.
 *
 * Clobbers A
 */
    ldh a, [$FF41]
    and %00000010
    jr nz, sub_vram_wait
    ret


version_string:
    db "{GBDOS_VERSION_STRING}", 0
message_string:
    db "Demo Kernel", 0
copyright_string:
    db "2023 Brayden Morris", 0
license_string:
    db "http://unlicense.org", 0

font_data:
incbin "BUILD/RES/TOMTHUMB.1BP"
font_data_end:

expansion_table:
    db %00000000
    db %00000011
    db %00001100
    db %00001111
    db %00110000
    db %00110011
    db %00111100
    db %00111111
